@@ -62,12 +62,15 @@ inline int getpagesize(void) {
 #include "settings/settings.hpp"
 #include "timevalues.h"
 #include "calibrate/CaliPara.hpp"
+#include "net/Connect.hpp"
+inline  constexpr const int maxdexcount=3025 ;
+inline constexpr const int youngsensorsecs=2*60*60;
 inline    constexpr const char rawstream[]="rawstream.dat";
 #include <string_view>
 extern std::string_view globalbasedir;
 //string basedir(FILEDIR);
 
-extern int writeStartime(crypt_t *pass, const int sock, const int sensorindex); 
+extern int writeStartime(crypt_t *pass,  Connect *connect, const int sensorindex); 
     extern int getgetsendnr();
 
 constexpr const int maxcaliNr=50;
@@ -717,16 +720,11 @@ uint32_t expectedEndTime() const {
      }
     
 uint32_t getmaxtime() const {
-/*
-    if(isLibre3()) {
-#ifndef NDEBUG
-        return 16*24*60*60+getstarttime();
-#else
-        return (14*24 + 2)*60*60+getstarttime();
-    #endif
-        }
-    else */
+#if  1
    const int hours= isSibionics1()?maxSIhours:((isAccuChek()?maxdaysAccu:getinfo()->days)*24);
+#else
+   const int hours= isSibionics1()?maxSIhours:((isAccuChek()?maxdaysAccu:getinfo()->days+1)*24);
+#endif
     return hours*60*60+getstarttime();
     }
 uint32_t getstarttime() const {
@@ -804,8 +802,10 @@ void saveel(int pos,time_t tin,uint16_t id, uint16_t const (&glu)[N]) {
 
 
 bool savenewhistory(int pos, int lifeCount, uint16_t mgL) {
-    if(!timelastcurrent)
+/*
+    if(!timelastcurrent) {
          return false;
+         } */
     Glucose *item=getglucose(pos);
     if(item->id==lifeCount&&item->glu[1]==mgL) {
 
@@ -1116,7 +1116,10 @@ static bool mkdatabase3(string_view sensordir,time_t start,uint32_t pin,const ch
                 return false;
             }
         }
-       Info inf{.starttime=(uint32_t)start,.lastscantime=(uint32_t)time(nullptr),.starthistory=0,.endhistory=0,.scancount=0,.startid=0,.interval=interval5,.dupl=3,.days=15 ,.pin=pin,.lastLifeCountReceived=1,.wearduration2=wearduration,.warmup2=static_cast<uint8_t>(warmup),.pollcount=0, .lockcount=0};
+       constexpr const int minperday=60*24;
+       const uint8_t  days=(wearduration+minperday-1)/minperday+1;
+       LOGGER("mkdatabase days=%d\n",days);
+       Info inf{.starttime=(uint32_t)start,.lastscantime=(uint32_t)time(nullptr),.starthistory=0,.endhistory=0,.scancount=0,.startid=0,.interval=interval5,.dupl=3,.days=days,.pin=pin,.lastLifeCountReceived=1,.wearduration2=wearduration,.warmup2=static_cast<uint8_t>(warmup),.pollcount=0, .lockcount=0};
     if(address)
         strcpy(inf.deviceaddress,address);
     else
@@ -1739,7 +1742,7 @@ CurData  scanInperiod(const uint32_t starttime,const uint32_t endtime) const {
 
 
 
-dataonlyptr  getfromfile(crypt_t *pass,int sock, std::string_view filename,int offset,int asklen) {
+dataonlyptr  getfromfile(crypt_t *pass,Connect *connect, std::string_view filename,int offset,int asklen) {
     const int pathlen=filename.size()+1;
     constexpr const int ali=alignof(struct askfile);
     const int comlen=((sizeof(askfile)+(pathlen+ali-1))/ali)*ali;
@@ -1751,13 +1754,13 @@ dataonlyptr  getfromfile(crypt_t *pass,int sock, std::string_view filename,int o
 
     ask->namelen=pathlen;
 
-    LOGGER("GLU: getfromfile(sock=%d %s offset=%d asklen=%d comlen=%d)\n",sock,filename.data(),offset,asklen,comlen);
+    LOGGER("GLU: getfromfile(sock=%d %s offset=%d asklen=%d comlen=%d)\n",connect->getSenderIdent(),filename.data(),offset,asklen,comlen);
     memcpy(ask->name,filename.data(),pathlen);
-    if(!noacksendcommand(pass,sock,buf,comlen) ) {
+    if(!connect->s_noacksendcommand(pass,buf,comlen) ) {
         LOGAR("GLU:  !noacksendcommand");
         return dataonlyptr(nullptr); 
         }
-    return receivedataonly(sock,pass, asklen);
+    return connect->receivedataonly_s(pass, asklen);
 
     };
 
@@ -1794,25 +1797,6 @@ int  getbackuptimescan(uint32_t starttime)  {
     LOGGER("GLU: getbackuptimescan pos=%d\n",pos);
     return pos;
     }
-    /*
-bool setbackuptimes(crypt_t *pass,int sock,int ind,uint32_t starttime) {
-    const int pathlen=infopath.size()+1;
-    constexpr const int ali=alignof(struct askfile);
-    const int comlen=((sizeof(askfile)+(pathlen+ali-1))/ali)*ali;
-    const int totlen=sizeof(askfile)+(pathlen);
-    const int comlen2=(ali-totlen%ali)+totlen;
-    assert(comlen==comlen2);
-    alignas(ali) uint8_t buf[comlen];
-    struct askfile *ask=reinterpret_cast<askfile *>(buf); 
-    const int asklen= offsetof(Info,pollinterval);
-    ask->com=saskfile;ask->off=0;ask->len=asklen;
-    memcpy(ask->name,infopath.data(),pathlen);
-    if(!noacksendcommand(pass,sock,buf,comlen) ) {
-        return false;
-        }
-       dataonlyptr res=receivedata(sock,pass, asklen) ;
-
-    } */
 uint32_t getbackuptimehistory(uint32_t starttime)  {
     int pos=gettimepos(starttime);
     const int endpos=getAllendhistory();
@@ -1850,12 +1834,12 @@ void backhistory(int pos) ;
 void backcalibrated(int pos);
 void backstream(int pos) ;
 
-bool setbackuptime(crypt_t *pass,int sock,int ind,uint32_t starttime) {
+bool setbackuptime(crypt_t *pass,Connect *connect,int ind,uint32_t starttime) {
     
     constexpr const int asklen= offsetof(Info,pollinterval);
     constexpr const int minlen= offsetof(Info,pollcount);
     LOGGER("GLU: %s setbackuptime %u asklen=%d\n",shortsensorname()->data(),starttime,asklen);
-    auto dontdestroy=getfromfile(pass,sock,infopath, 0,asklen);
+    auto dontdestroy=getfromfile(pass,connect,infopath, 0,asklen);
    dataonly *dat=dontdestroy.get();
     if(dat==nullptr) {
         LOGSTRING("GLU: ==nullptr\n");
@@ -1873,10 +1857,10 @@ bool setbackuptime(crypt_t *pass,int sock,int ind,uint32_t starttime) {
             uint32_t scanstart=
 #endif
             getinfo()->update[ind].scanstart=(scantime<starttime)?scanend:getbackuptimescan(starttime);
-            LOGGER("GLU: scanstart=%d scancount=%d\n",scanstart,scanend);
+            LOGGER("GLU: ind=%d scanstart=%d scancount=%d\n",ind,scanstart,scanend);
             }
         else {
-            LOGGER("GLU scanstart=%d\n",getinfo()->update[ind].scanstart);
+            LOGGER("GLU ind=%d scanstart=%d\n",ind,getinfo()->update[ind].scanstart);
             }
         if(getinfo()->update[ind].histstart==0) { //TODO why only when it is zero?
         //Why not just the from the position with that time here?
@@ -1889,10 +1873,10 @@ bool setbackuptime(crypt_t *pass,int sock,int ind,uint32_t starttime) {
 #endif
             getinfo()->update[ind].histstart= (histpos=posearlier(histend,starttime)>=0)?histpos:getbackuptimehistory(starttime);
 
-            LOGGER("GLU: hist start=%d endhistory=%d\n",histstart,histend);
+            LOGGER("GLU: ind=%d hist start=%d endhistory=%d\n",ind,histstart,histend);
             }
         else {
-            LOGGER("GLU histstart=%d\n",getinfo()->update[ind].histstart);
+            LOGGER("GLU ind=%d histstart=%d\n",ind,getinfo()->update[ind].histstart);
             }
         if(getinfo()->update[ind].streamstart==0) {
 #ifndef NOLOG
@@ -1902,10 +1886,10 @@ bool setbackuptime(crypt_t *pass,int sock,int ind,uint32_t starttime) {
             uint32_t streamstart=
 #endif
             getinfo()->update[ind].streamstart=getbackuptimestream(starttime);
-            LOGGER("GLU: %s streamstart=%d streamend=%d\n", showsensorname().data(),streamstart,streamend);
+            LOGGER("GLU: ind=%d %s streamstart=%d streamend=%d\n", ind,showsensorname().data(),streamstart,streamend);
             }
         else {
-            LOGGER("GLU: %s streamstart=%d\n", showsensorname().data(),getinfo()->update[ind].streamstart);
+            LOGGER("GLU: ind=%d  %s streamstart=%d\n",ind, showsensorname().data(),getinfo()->update[ind].streamstart);
             }
         }
     return true;
@@ -1969,7 +1953,7 @@ void setsiScan(int maxind) {
         }
     }
 
-int updateKAuth(crypt_t *pass,int sock,int ind);
+int updateKAuth(crypt_t *pass,Connect *connect,int ind);
 void setsendhiststart() {
     const int maxind=getgetsendnr();
     updatestate  *up= getinfo()->update;
@@ -1984,7 +1968,7 @@ sendscan:
 2: also via scan
 */
 private:
-int sendSistate(const pathconcat *sfile, crypt_t *pass,int sock)  {
+int sendSistate(const pathconcat *sfile, crypt_t *pass,Connect *connect)  {
     const char *statename=sfile->data();
     Readall<unsigned char> stateBytes(statename);
     if(!stateBytes.data()||!stateBytes.size()) {
@@ -1993,17 +1977,17 @@ int sendSistate(const pathconcat *sfile, crypt_t *pass,int sock)  {
         }
     const auto relstate=absToRel(*sfile);
     LOGGER("statefile=%s\n",statename);
-    if(!senddata(pass,sock,0,stateBytes.data(),stateBytes.size(),relstate)) {
+    if(!connect->senddata(pass,0,stateBytes.data(),stateBytes.size(),relstate)) {
         LOGGER("GLU: senddata %s failed\n",relstate.data());
         return 0;
         }
     return 1;
     }
 public:
-int sendSibionicsState(crypt_t *pass,int sock,int ind)  {
+int sendSibionicsState(crypt_t *pass,Connect *connect,int ind)  {
     int waslock=getinfo()->lockcount;
     if(getinfo()->update[ind].rawstreamstart<waslock) {
-        int res=sendSistate(&binstatefile,pass,sock);
+        int res=sendSistate(&binstatefile,pass,connect);
         /*
         if(res==2) {
             res=sendSistate(&statefile,pass,sock);
@@ -2016,10 +2000,10 @@ int sendSibionicsState(crypt_t *pass,int sock,int ind)  {
     return 2;
     }
 
-int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
+int updatestream(crypt_t *pass,Connect *connect,int ind,int sensindex,int sendscan)  {
     getinfo()->update[ind].changedstreamstart=false;
     int streamstart=getinfo()->update[ind].streamstart;
-    LOGGER("updatestream sock=%d ind=%d sensindex=%d streamstart=%d sendscan=%d\n",sock,ind,sensindex,streamstart,sendscan);
+    LOGGER("updatestream sock=%d ind=%d sensindex=%d streamstart=%d sendscan=%d\n",connect->getSenderIdent(),ind,sensindex,streamstart,sendscan);
     struct {
         uint32_t pollcount;
         double pollinterval; 
@@ -2036,11 +2020,11 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
 #ifndef NOLOG
         const struct ScanData *fn=startstreambuf+streamstart;
         time_t tim=fn->t;
-        LOGGER("GLU: streamstart=%d streamend=%d %s %.1f (%d) dowith=%d %s",streamstart,streamend,polluit.data(),fn->g/convfactordL,fn->g,cmd,ctime(&tim));
+        LOGGER("GLU: ind=%d streamstart=%d streamend=%d %s %.1f (%d) dowith=%d %s",ind,streamstart,streamend,polluit.data(),fn->g/convfactordL,fn->g,cmd,ctime(&tim));
 #endif
 
-        if(!senddata(pass,sock,streamstart,startstreambuf+streamstart,startstreambuf+streamend,polluit)) {
-            LOGSTRING("GLU: senddata polls.dat failed\n");
+        if(!connect->senddata(pass,streamstart,startstreambuf+streamstart,startstreambuf+streamend,polluit)) {
+            LOGGER("GLU: ind=%d senddata %s failed\n",ind,polluit.data());
             return 0;
             }
 
@@ -2054,7 +2038,7 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
         switch(sendscan) {
             case 1: {
                 memcpy(&endinfo,&getinfo()->endStreamhistory,sizeof(endinfo));
-                int res=newsendhistory(pass,sock,ind,sensindex, true,endinfo.endStreamhistory) ;
+                int res=newsendhistory(pass,connect,ind,sensindex, true,endinfo.endStreamhistory) ;
                 switch(res) {
                     case 0: return 0;
                     case 1: {
@@ -2069,7 +2053,7 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
                 };break;
             case 2: {
                 memcpy(&endinfo,&getinfo()->endStreamhistory,sizeof(endinfo));
-                int res=oldsendhistory(pass,sock,ind, sensindex,false,endinfo.endStreamhistory) ;
+                int res=oldsendhistory(pass,connect,ind, sensindex,false,endinfo.endStreamhistory) ;
                 switch(res) {
                     case 0: return 0;
                     case 1: {
@@ -2125,19 +2109,19 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
             }
 
             vect.push_back({reinterpret_cast<const senddata_t *>(&getinfo()->broadcastfrom),offsetof(Info,broadcastfrom),sizeof(getinfo()->broadcastfrom)});
-         if(!senddata(pass,sock,vect, infopath,cmd,reinterpret_cast<const uint8_t *>(&streamstart),sizeof(streamstart))) {
+         if(!connect->senddata(pass,vect, infopath,cmd,reinterpret_cast<const uint8_t *>(&streamstart),sizeof(streamstart))) {
             LOGSTRING("GLU: senddata info.data failed\n");
             return 0;
             }
         if(updateStarttime) {
-            if(!writeStartime(pass,sock,sensindex))  {
+            if(!writeStartime(pass,connect,sensindex))  {
                 return 0;
                 }
             getinfo()->update[ind].siStream=true;
             }
 
         if(!getinfo()->update[ind].changedstreamstart) {
-            LOGGER("streamstart=%d\n",streamend);
+            LOGGER("ind=%d updatestream new streamstart=%d\n",ind,streamend);
             getinfo()->update[ind].streamstart=streamend;
 
             }
@@ -2147,7 +2131,7 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
         if(sendhiststart) getinfo()->update[ind].sendhiststart=false;
         if(isLibre3()||isDexcom()) {
             int endhistory=getScanendhistory();    
-            if(oldsendhistory(pass,sock,ind,sensindex,true,endhistory))
+            if(oldsendhistory(pass,connect,ind,sensindex,true,endhistory))
                 return 1;
             return 0;
             }
@@ -2157,7 +2141,7 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
     else {
         if(getinfo()->update[ind].sendbluetoothOn) {
               constexpr    const int offset=offsetof(Info,streamingIsEnabled);
-              if(!senddata(pass,sock,offset,meminfo.data()+offset,1, infopath)) {
+              if(!connect->senddata(pass,offset,meminfo.data()+offset,1, infopath)) {
                 LOGSTRING("GLU: senddata bluetoothON info.data failed\n");
                       return 0;
                    }
@@ -2171,20 +2155,26 @@ int updatestream(crypt_t *pass,int sock,int ind,int sensindex,int sendscan)  {
 private:
 
 
-int sendhistoryinfo(crypt_t *pass,int sock,int sensorindex,uint32_t histstart,uint32_t endhistory) ;
+int sendhistoryinfo(crypt_t *pass,Connect *connect,int sensorindex,uint32_t histstart,uint32_t endhistory) ;
 //int sendhistory(crypt_t *pass,int sock,int ind,int sendindex,bool) ;
-int oldsendhistory(crypt_t *pass,int sock,int ind,int sensorindex,bool sendinfo,int histend) ;
-int newsendhistory(crypt_t *pass,int sock,int ind,int sensorindex,bool sendStream,int histend) ;
+int oldsendhistory(crypt_t *pass,Connect *connect,int ind,int sensorindex,bool sendinfo,int histend) ;
+int newsendhistory(crypt_t *pass,Connect *connect,int ind,int sensorindex,bool sendStream,int histend) ;
 
 public:
-int updatescan(crypt_t *pass,int sock,int  ind,int sensorindex,bool,int sendstream); 
-int updatescanalg(crypt_t *pass,int sock,int  ind,int sensorindex,int alsostream) { 
-    return updatescan(pass,sock,ind,sensorindex,false,alsostream)&3; 
+int updatescan(crypt_t *pass,Connect *connect,int  ind,int sensorindex,bool,int sendstream); 
+int updatescanalg(crypt_t *pass,Connect *connect,int  ind,int sensorindex,int alsostream) { 
+    return updatescan(pass,connect,ind,sensorindex,false,alsostream)&3; 
     }
 int lastlifecount=0;
 time_t timelastcurrent=0;
 time_t lifeCount2time(uint32_t lifecount) {
-    auto uit=timelastcurrent-60LL*(lastlifecount-(int64_t)lifecount);
+    time_t uit;
+    if(lastlifecount) {
+        uit=timelastcurrent-60LL*(lastlifecount-(int64_t)lifecount);
+        }
+    else {
+        uit=getstarttime()+60LL*(lifecount+1);
+        }
     LOGGER("timelastcurrent=%ld lastlifecount=%d lifeCount2time(lifecount=%d)=%lld\n", timelastcurrent,lastlifecount,lifecount,uit);
     return uit;
     }
@@ -2250,7 +2240,7 @@ int siAddedIndex(int index) const {
 
 void resetSiIndex();
 
-int updateCali(crypt_t *pass,int sock,int ind,int sensorindex)  {
+int updateCali(crypt_t *pass,Connect *connect,int ind,int sensorindex)  {
      uint32_t wascaliNr=getinfo()->caliNr;
      uint32_t updatefrom=getinfo()->caliUpdated[ind];
      if(updatefrom==wascaliNr)
@@ -2268,11 +2258,11 @@ int updateCali(crypt_t *pass,int sock,int ind,int sensorindex)  {
         }
      vect.push_back({reinterpret_cast<const senddata_t *>(&wascaliNr),offsetof(Info,caliNr),4});
     const uint16_t calibratedstartcmd=startcalibratedupdate|sensorindex;
-    if(!senddata(pass,sock,vect, infopath,calibratedstartcmd, reinterpret_cast<const uint8_t *>(&updatefrom),sizeof(updatefrom))) {
+    if(!connect->senddata(pass,vect, infopath,calibratedstartcmd, reinterpret_cast<const uint8_t *>(&updatefrom),sizeof(updatefrom))) {
       LOGAR("updateCali: senddata info.data failed");
       return 0;
      }
-     LOGGER("updateCali sock=%d ind=%d sensorindex=%d caliNR=%u updatefrom=%u\n",sock,ind,sensorindex,wascaliNr,updatefrom);
+     LOGGER("updateCali sock=%d ind=%d sensorindex=%d caliNR=%u updatefrom=%u\n",connect->getSenderIdent(),ind,sensorindex,wascaliNr,updatefrom);
      getinfo()->caliUpdated[ind]=wascaliNr;
      return 1;
  }
@@ -2290,6 +2280,37 @@ int getLastIndex() const {
                 }
         return -1;
         }
+
+
+bool hasData(uint32_t nu) const {
+        if(isAccuChek()) {
+            if(pollcount()>=4000)
+                    return false;
+            }
+       else {
+        if(isDexcom()) {
+            if(pollcount()>=maxdexcount)
+                    return false;
+            }
+          else {
+          /*
+             if(isLibre3()) {
+                    if(pollcount()>=getinfo()->wearduration2)
+                        return false;
+                    }
+              else
+              */
+              {
+                    return false;
+                    }
+              }
+           }
+    const uint32_t diff=nu-lastused();
+    const bool res= diff < youngsensorsecs;
+    LOGGER("hasData nu=%d lastused=%d diff=%d return=%d\n",nu,lastused(),diff,res);
+     return res;
+    }
+//         return ((isAccuChek()&&pollcount()<4000)||(isDexcom()&&pollcount()<maxdexcount))&& (nu-lastused())< youngsensorsecs
 };
 
 struct lastscan_t {
