@@ -155,6 +155,8 @@ jugglucotext betext {
 .libre3scansuccess= {"FreeStyle Libre 3 датчык", "Цяпер Juggluco будзе атрымліваць значэнні глюкозы"},
 .unknownNFC={"Памылка нераспазнанага сканавання NFC", "Паспрабуй яшчэ"},
 .nolibre3={"FreeStyle Libre 3 датчык","Не падтрымліваецца гэтай версіяй Juggluco"},
+.libre3zeroID={"Error, zero account ID?",
+	R"(Use Left menu->Settings->Exchange data->Libreview->"Get Account ID" to set a non-zero account ID.)"},
 .needsandroid8="Needs minimally Android 8"sv,
 #ifndef WEAROS
 	.advancedstart= R"(<h1>Мадыфікаванае прылада</h1>
