@@ -36,17 +36,21 @@
 #include <stdlib.h>
 
 #include <jni.h>
+#include "net/TCPConnect.hpp"
 #include "myfdsan.h"
+
+#include "logvector.hpp"
 inline int mytag() {
    return 0;
    }
 
+extern void  startReceiverThread(int i);
 extern void setBlueMessage(int ident,bool val);
 constexpr const char defaultport[]{
-#ifdef RELEASEID
-"8795"
+#ifdef MIRRORPORT
+xquotes(MIRRORPORT)
 #else
-"9113"
+"9996"
 #endif
 
 };
@@ -66,19 +70,13 @@ constexpr const char defaultport[]{
 #ifdef WEAROS_MESSAGES
 extern bool wearmessages[];
 #endif
-/*
-inline void closesock(int &sock) {
-    if(sock>=0) {
-        LOGGER("%d ",sock);
-        ::shutdown(sock,SHUT_RDWR);
-        sock=-1;
-        }
-    }
-    */
 #ifndef TESTMENU
 #include <mutex>
 extern std::mutex change_host_mutex;
 #endif
+
+extern void setConnectTime(const int allindex,uint32_t tim);
+extern Connect *connections[];
 extern bool networkpresent;
 class Backup;
 extern Backup *backup;
@@ -103,11 +101,11 @@ struct changednums {
         lastlastpos=0;
         }
     };
-int  updatenums(crypt_t *,int sock,struct changednums *nums);
+int  updatenums(crypt_t *,Connect *connect,struct changednums *nums);
 inline static constexpr const char backupdat[]="backup.dat";
 inline static constexpr const char orbackup[] ="orbackup.dat";
-extern int hostsocks[];
-extern std::vector<int> sendsocks;
+//extern int hostsocks[];
+//extern std::vector<int> sendsocks;
 extern uint32_t lastuptodate[];
 
 //extern int tmpsocks[maxallhosts][passhost_t::maxip];
@@ -161,25 +159,40 @@ struct updateone {
         }
     
     crypt_t *getcrypt() const; 
+
+    template <typename Self>
+auto *getConnect(this Self&&self)  {
+        return connections[self.allindex];
+        }
     int &getsock() const {
-        return sendsocks[ind];
+        auto *con=((TCPConnect *)getConnect());
+        if(!con) {
+                static int minone=-1;
+                return minone;
+                }
+//        return sendsocks[ind];
+        return con->getSenderSock();
         }
     void setsock(int val) const {
-        sendsocks[ind]=val;
+        if(auto *con=((TCPConnect *)getConnect())) {
+                con->setSenderSock(val);
+          } 
+      //  sendsocks[ind]=val;
         }
 void    open() ;
 void close() {    
-       const int so= getsock();
-       LOGGER("updateone close(%d)\n",so);
-       if(so>=0) {
-//        mirrorstatus[allindex].sensor.hassocket=false;
+    auto *con =connections[allindex];
+    if(con) {
+        con->closeSenderConnection();
+//       const int so= getsock();
+       LOGGER("updateone close(%d)\n",con->getSenderIdent());
+       }
+    /*   if(so>=0) {
          setsock(-1);
-//         if(get_owner_tag(so)==mytag()) 
-            ::sockclose(so);
-         if(crypt_t *ctx=getcrypt()) {
-//            LOGGER("ascon_aead_cleanup(%p)\n",ctx);
-            ascon_aead_cleanup(ctx);
-            }
+         ::sockclose(so);
+        } */
+     if(crypt_t *ctx=getcrypt()) {
+        ascon_aead_cleanup(ctx);
         }
     }
 
@@ -194,6 +207,12 @@ int sendCalibrate();
     };
 
 #include "maxsendtohost.h"
+struct TurnServer {
+    uint16_t port;
+    char hostname[254];
+    char username[256];
+    char password[256];
+    };
 struct updatedata {
     int32_t hostnr;
     uint32_t receive;
@@ -203,45 +222,56 @@ struct updatedata {
     uint8_t sendnr;
     passhost_t allhosts[maxallhosts];
     updateone tosend[maxsendtohost];
-
+    uint32_t lastused[maxallhosts];
+    int32_t NRturnserver;
+    TurnServer  turnserver[4];
     void wakesender() ;
     void wakestreamsender();
     };
 
-class  Backup {
-
-Mmap<unsigned char> mapdata;
-public:
+static constexpr const uintptr_t wakestream=1;
+static constexpr const uintptr_t wakescan=2;
+static constexpr const uintptr_t wakenums=4;
+static constexpr const uintptr_t wakeall=8;
+static constexpr const uintptr_t wakestop=16;
+static constexpr const uintptr_t wakeother=32;
+static constexpr const uintptr_t wakeend=64;
+static constexpr const uintptr_t wakesend=128;
+static constexpr const uintptr_t wakereconnect=256;
+static constexpr const uintptr_t wakestreamsend=512;
 struct  condvar_t {
     uintptr_t dobackup=0;
     std::mutex backupmutex;
     std::condition_variable backupcond; 
-    /*
-void setkind(uintptr_t kind){
-       if(dobackup)
-           dobackup=kind;
-      }
-      */
-void wakebackuponly(uintptr_t kind){
-     std::lock_guard<std::mutex> lck(backupmutex);
-     dobackup=kind|(dobackup&(wakeend|wakestop));
-     backupcond.notify_one();                        
-     }
-void wakebackup(uintptr_t kind){
-        {
-          std::lock_guard<std::mutex> lck(backupmutex);
-          auto wasdo=dobackup;
-          dobackup=dobackup|kind;
-        if(wasdo) {
-                   LOGGER("wakebackup %lx  no-notify\n",kind);
-            return;
-            }
 
-        LOGGER("wakebackup %lx notify\n",kind);
-        }
-        backupcond.notify_one();                        
-      }
+    void wakebackuponly(uintptr_t kind){
+         std::lock_guard<std::mutex> lck(backupmutex);
+         dobackup=kind|(dobackup&(wakeend|wakestop));
+         backupcond.notify_one();                        
+         }
+    void wakebackup(uintptr_t kind){
+            {
+              std::lock_guard<std::mutex> lck(backupmutex);
+              auto wasdo=dobackup;
+              dobackup=dobackup|kind;
+            if(wasdo) {
+                       LOGGER("wakebackup %lx  no-notify\n",kind);
+                return;
+                }
+
+            LOGGER("wakebackup %lx notify\n",kind);
+            }
+            backupcond.notify_one();                        
+          }
     };
+
+extern condvar_t* active_receive[maxallhosts];
+extern int active_receivenr;
+class  Backup {
+
+Mmap<unsigned char> mapdata;
+public:
+
 std::vector<condvar_t*> con_vars;
 struct updatedata* getupdatedata() {return reinterpret_cast<struct updatedata*>(mapdata.data());}
 const struct updatedata* getupdatedata() const {return reinterpret_cast<const struct updatedata*>(mapdata.data());}
@@ -286,7 +316,10 @@ void sendNightNumbers() {
 void startactivereceivers() {
     const int len=getupdatedata()->hostnr;
     for(int i=0;i<len;i++) {
-        hostsocks[i]=-1;
+/*        if(backup) {
+            if(auto *con=connections[i])
+                con->closeReceiverConnection();
+            } */
         passhost_t *ph= getupdatedata()->allhosts+i;
         if(ph->activereceive&&!ph->deactivated) {
             setactivereceive(i,ph); 
@@ -309,68 +342,7 @@ void resetindices() {
     LOGGER("sendnr=%d\n", getupdatedata()->sendnr);
     }
 
-Backup(std::string_view base): mapdata(base,backupdat,sizeof(struct updatedata)), con_vars((resetindices(),getupdatedata()->sendnr)) {
-   const int len=getupdatedata()->hostnr;
-   sendsocks.reserve(getupdatedata()->sendnr);
-   crypts.reserve(getupdatedata()->sendnr);
-   for(int i=0;i<getupdatedata()->sendnr;i++) {
-       sendsocks.push_back(-1);
-       auto &host=getupdatedata()->tosend[i];
-       if(settings->data()->initVersion<31) { 
-          LOGGER("%d set sendjugglucoid=false\n",i);
-          host.sendjugglucoid=false;
-#ifndef WEAROS
-          if(getupdatedata()->allhosts[host.allindex].wearos) {
-              if(settings->data()->sendnumbers)
-                    host.sendLibre=true;
-              if(settings->data()->saytreatments ||settings->data()->postTreatments)
-                    host.sendNight=true;
-            }
-#endif
-          }
-       if(getupdatedata()->allhosts[host.allindex].haspass()) {
-           auto cry=new crypt_t();
-           LOGGER("crypts[%d]=%p=new crypt_t()\n",i,cry);
-           crypts.push_back(cry);
-           }
-       else  {
-           LOGGER("crypts[%d]=nullptr\n",i);
-           crypts.push_back(nullptr);
-           }
-       }
-
-
-
-    startactivereceivers();
-    /*
-   for(int i=0;i<len;i++) {
-       hostsocks[i]=-1;
-       passhost_t *ph= getupdatedata()->allhosts+i;
-       if(ph->activereceive) {
-           setactivereceive(i,ph); 
-           }
-       } */
-   if(!getupdatedata()->port[0])
-       strcpy(getupdatedata()->port,defaultport );
-
-   void    backupbase(string_view basedir);
-   backupbase(globalbasedir);
-   for(int i=0;i<len;i++) {
-       if(getupdatedata()->allhosts[i].passive()) {
-           startreceiver(false);
-           break;
-           }
-       }
-
-
-   shouldaskfordata=getshouldaskfordata();
-   /*
-   #ifdef WEAROS_MESSAGES
-   extern void startmessagereceivers(Backup*);
-   if(wearmessages)
-       startmessagereceivers(this);
-   #endif */
-   }
+Backup(std::string_view base);
 
 void resensordata(int sensor)  {
      if(sensor<0) {
@@ -412,17 +384,21 @@ void startreceiver(bool always=false) {
     if(receiveractive()&&!always)
         return;
     ::stopreceiver();
-    ::startreceiver(getupdatedata()->port,getupdatedata()->allhosts,getupdatedata()->hostnr,hostsocks);
+    ::startreceiver(getupdatedata()->port,getupdatedata()->allhosts,getupdatedata()->hostnr);
     }
 void stopreceiver() {
     ::stopreceiver() ;
     for(int i=0;i<getupdatedata()->hostnr;i++) {
         if(getupdatedata()->allhosts[i].receivefrom) {
+            if(auto *con=connections[i])
+                con->shutdownReceiver();
+            /*
             int sock=hostsocks[i];
             if(sock>=0) {
                 LOGGER("shutdown %d\n",sock);
                 ::shutdown(sock,SHUT_RDWR);
                 }
+                */
             }
         }
 
@@ -449,7 +425,6 @@ void getport(int pos,char *buf) {
         strcpy(buf,defaultport);
     else
         snprintf(buf,6, "%d",port);
-//    snprintf(buf,6, "%d",ntohs(host.ips[0].sin6_port));
     }
 
 void addsize() {
@@ -474,11 +449,14 @@ void deletestart(int sendindex) {
                 int sin=host.index;
                 if(sin>=sendindex) {
                        if(con_vars[sin]) {
-                           const int sock= getupdatedata()->tosend[sin].getsock();
+//                           const int sock= getupdatedata()->tosend[sin].getsock();
                            LOGGER("call wakestop %p\n",con_vars[sin]);
-                           con_vars[sin]->wakebackuponly(Backup::wakestop);
-                           LOGGER(" shutdown %d\n",sock);
-                           ::shutdown(sock,SHUT_RDWR);
+                           con_vars[sin]->wakebackuponly(wakestop);
+                            if(auto *con=connections[i]) {
+                               LOGGER(" shutdown %d\n",con->getSenderIdent());
+                               con->shutdownSender();
+                               }
+//                           ::shutdown(sock,SHUT_RDWR);
                            }
                        }
                  }
@@ -492,7 +470,7 @@ void deletestart(int sendindex) {
             LOGGER("crypts[%d]=nullptr\n", getupdatedata()->sendnr);
             }
         if(con_vars[getupdatedata()->sendnr])
-           con_vars[getupdatedata()->sendnr]->wakebackuponly(Backup::wakestop|Backup::wakeend);
+           con_vars[getupdatedata()->sendnr]->wakebackuponly(wakestop|wakeend);
         }
 void deleteend(int sendindex) {
         for(int i=0;i<backup->getupdatedata()->hostnr;i++) {
@@ -515,23 +493,21 @@ void setindices(int start) {
 void deletehost(int index) {
     if(index>=getupdatedata()->hostnr)
         return;
+    bool wasnet=networkpresent;
+     networkpresent=false;
     LOGGER("deletehost(%d)\n",index);
+    for(int it=index;it<getupdatedata()->hostnr;++it) {
+         auto *con=connections[it];
+         if(con) {
+            con->finish=true;
+            con->endConnection();
+            }
+        }
     auto &thehost=getupdatedata()->allhosts[index];
     if(thehost.activereceive) {
         endactivereceive(index); 
         }
-    if(getupdatedata()->allhosts[index].receivefrom) {
-        int sock=hostsocks[index];
-        if(sock>=0) {
-            LOGGER("%d: close(%d)\n",index,sock);
-            ::shutdown(sock,SHUT_RDWR);
-        // if(get_owner_tag(sock)==mytag()) 
-            ::sockclose(sock);
-            hostsocks[index]=-1;
-            }
-        }
-    bool wasnet=networkpresent;
-     networkpresent=false;
+    sleep(1);
     const int sendindex=getupdatedata()->allhosts[index].index;
     if(sendindex>=0) {
         deletestart(sendindex);
@@ -540,8 +516,10 @@ void deletehost(int index) {
         LOGGER("--hostnr %d\n",getupdatedata()->hostnr);
         int fromend= getupdatedata()->hostnr-index;
         memmove( getupdatedata()->allhosts+index,getupdatedata()->allhosts+index+1,fromend*sizeof(getupdatedata()->allhosts[0]));
-        memmove(  hostsocks+index, hostsocks+index+1,fromend*sizeof(hostsocks[0]));
+        memmove( getupdatedata()->lastused+index,getupdatedata()->lastused+index+1,fromend*sizeof(getupdatedata()->lastused[0]));
+//        memmove(  hostsocks+index, hostsocks+index+1,fromend*sizeof(hostsocks[0]));
         memmove(  lastuptodate+index, lastuptodate+index+1,fromend*sizeof(lastuptodate[0]));
+        memmove(  connections+index, connections+index+1,fromend*sizeof(connections[0]));
         }
 
     if(sendindex>=0)      {
@@ -553,18 +531,26 @@ void deletehost(int index) {
         getupdatedata()->sendnr=0;
         stopreceiver();
         }
+    delete connections[getupdatedata()->hostnr];
+    connections[getupdatedata()->hostnr]=nullptr;
     setindices(index);
+    for(int i=index;i<getupdatedata()->hostnr;++i) {
+        connections[i]->finish=false;
+        connections[i]->setindex(i);
+        }
      if(!networkpresent)
         networkpresent=wasnet;
     }
 void clearhost(int index) {
     LOGGER("clearhost(%d)\n",index);
     updateone &host=getupdatedata()->tosend[index];
+    /*
     if(int so=host.getsock();so!=-1) {
         LOGGER("shutdown(%d)\n",so);
         ::shutdown(so,SHUT_RDWR);
-        }
-
+        } */
+    if(auto *con=connections[host.allindex])
+        con->restartSender();
     sensors->updateinit(index);
 
     host.startsensors=0;
@@ -605,7 +591,7 @@ void changereceiver(int allindex,int index,const bool sendnums,const bool sendst
     LOGGER("changereceiver(allindex=%d,index=%d,sendnums=%d,sendstream=%d,sendscans=%d,haspass=%d,starttime=%u) sendnr=%d\n",allindex,index,sendnums,sendstream,sendscans,haspass,starttime,getupdatedata()->sendnr);
     if(index==getupdatedata()->sendnr) {
         addsize();
-        sendsocks.resize(getupdatedata()->sendnr+1,-1);
+      //  sendsocks.resize(getupdatedata()->sendnr+1,-1);
 //        tmpsocks.resize(getupdatedata()->sendnr+1);
         crypts.resize(getupdatedata()->sendnr+1,nullptr);
         clearhost(index);
@@ -614,7 +600,9 @@ void changereceiver(int allindex,int index,const bool sendnums,const bool sendst
         }
     else  {
         if(con_vars[index])
-            con_vars[index]->wakebackuponly(Backup::wakestop);
+            con_vars[index]->wakebackuponly(wakestop);
+        if(auto *con=connections[allindex])
+                con->shutdownSender();
         }
     updateone &host=getupdatedata()->tosend[index];
     host.setindex(index,allindex);
@@ -628,11 +616,12 @@ void changereceiver(int allindex,int index,const bool sendnums,const bool sendst
     host.sendstream=sendstream;
     host.sendscans=sendscans;
     host.restore=restore;
+/*
     if(int so=host.getsock();so!=-1) {
         LOGGER("shutdown(%d)\n",so);
         ::shutdown(so,SHUT_RDWR);
     //    host.setsock(-1);
-        }
+        } */
     crypt_t *oldcrypt=host.getcrypt();
     if(haspass)  {
         if(!oldcrypt)  {
@@ -758,34 +747,50 @@ int setips(const char *port, const char **names, const int len,struct sockaddr_i
        case -1: mess="Port should between 1024 and 65535";break;
                                 case -2: mess="Error parsing ip";break;
                 */
-
 void endactivereceive(int allindex) { 
-      LOGGER("endactivereceive(%d)\n",allindex);
-    extern std::vector<condvar_t*> active_receive;
+    LOGGER("endactivereceive(%d)\n",allindex);
     passhost_t *ph=getupdatedata()->allhosts+allindex;
     if(ph->activereceive) {
         if(!ph->deactivated) {
-            if(auto actrec=active_receive[ph->activereceive-1])
-                actrec->wakebackup(Backup::wakeend);
+            int pos=ph->activereceive-1;
+            if(auto actrec=active_receive[pos]) {
+                LOGGER("wakeend %d\n",pos);
+                actrec->wakebackup(wakeend);
+                }
             }
+        if(ph->activereceive==active_receivenr)
+            --active_receivenr;
+        
         ph->activereceive=0;
-        LOGGER("endactivereceive(%d) shutdown(%d)\n",allindex,hostsocks[allindex]);
-        ::shutdown(hostsocks[allindex],SHUT_RDWR);
+        if(Connect *con=connections[allindex]) {
+            LOGGER("endactivereceive(%d) shutdown(%d)\n",allindex,con->getReceiverIdent());
+
+            con->shutdownReceiver();
+            }
+//        ::shutdown(hostsocks[allindex],SHUT_RDWR);
         }
     }
 void setactivereceive(int allindex,passhost_t *ph,bool startthread=true) { 
-       extern std::vector<condvar_t*> active_receive;
-       active_receive.push_back(new condvar_t);
-       ph->activereceive=active_receive.size();
-       LOGGER("setactivereceive allindex=%d nr=%d\n",allindex,ph->activereceive);
+       if(active_receivenr<maxallhosts) {
+           active_receive[active_receivenr++]=new condvar_t;
+           ph->activereceive=active_receivenr;
 
-       if(startthread) {
-           void activereceivethread(int allindex,passhost_t *pass) ;
-           std::thread the(activereceivethread,allindex,ph);
-           the.detach();
-           }
+           LOGGER("setactivereceive allindex=%d nr=%d %p\n",allindex,ph->activereceive,active_receive[ph->activereceive-1]);
+
+           if(startthread) {
+               void activereceivethread(int allindex,passhost_t *pass) ;
+               std::thread the(activereceivethread,allindex,ph);
+               the.detach();
+               }
+            }
+        else {
+            LOGGER("setactivereceive active_receivenr too large %d\n", active_receivenr);
+            kill(getpid(),15);
+            }
        }
 
+
+int changeICEhost(const char *ICElabel,int index,const bool sendnums,const bool sendstream,const bool sendscans,const bool receive,string_view pass,uint32_t starttime,const char *label,bool side,bool startthreads=true);
 int changehost(int index,JNIEnv *env,jobjectArray jnames,int nr,bool detect,string_view port,const bool sendnums,const bool sendstream,const bool sendscans,const bool restore,const bool receive,const bool activeonly,string_view pass,uint32_t starttime,bool passiveonly,const char *label=nullptr,const bool testip=true,bool startthreads=true,bool hashostname=false) {
     const int hostnr=getupdatedata()->hostnr;
     LOGGER("hostnr=%d changehost(%d,sendnums=%d,sendstream=%d,sendscans=%d,receive=%d,activeonly=%d,passiveonly=%d,label=%s port=%s nr=%d hashostname=%d\n",hostnr,index,sendnums,sendstream,sendscans,receive,activeonly,passiveonly,label,port.data(),nr,hashostname);
@@ -836,7 +841,6 @@ int changehost(int index,JNIEnv *env,jobjectArray jnames,int nr,bool detect,stri
             tohost=getupdatedata()->sendnr;
             if(tohost>=maxsendtohost) {
                 LOGGER("changehost: tohost(%d)>=maxsendtohost(%d)\n",tohost,maxsendtohost);
-            //getupdatedata()->sendnr=std::min(maxsendtohost,hostnr);
                 return -4;
                 }
             thehost.index=tohost;
@@ -863,7 +867,22 @@ int changehost(int index,JNIEnv *env,jobjectArray jnames,int nr,bool detect,stri
             }
         thehost.index=-1;
         }
+    if(!newhost) {  
+        if(auto *con=connections[index]) {
+            if(!thehost.ICE) {
+                con->setindex( index);
+                goto keepTCP;
+                }
+            con->finish=true;
+            con->endConnection();
+            sleep(1);
+            delete con;
+            }
+        }
 
+    connections[index]=new TCPConnect(index);
+    thehost.ICE=false;
+    keepTCP:
     if(!newhost&&thehost.activereceive)
         endactivereceive(index) ;
     int res;
@@ -940,10 +959,10 @@ int changehost(int index,JNIEnv *env,jobjectArray jnames,int nr,bool detect,stri
     lastuptodate[index]=0;
 /*
 Receive           reconnect    receivefrom:
-true        true           3
-true          false          2
-false        true          1
-false          false          0
+true              true           3
+true              false          2
+false             true           1
+false             false          0
 */
     thehost.receivefrom=receive?(reconnect?3:2):((sendto&reconnect)?1:0);
     LOGGER("changehost receivefrom=%d\n", thehost.receivefrom);
@@ -960,6 +979,7 @@ false          false          0
         LOGGER("wearos(%d)=%d\n", index,thehost.wearos);
         }
 
+    setConnectTime(index,0);
     deupdated(); 
 
     closesocksone(index,getupdatedata()->allhosts+index);
@@ -971,19 +991,31 @@ false          false          0
         }
     LOGGER("activereceive was=%d nu=%d\n",thehost.activereceive,receiveactive);
     if(receiveactive) {
-        setactivereceive(index,getupdatedata()->allhosts+index,startthreads);
+        if(newhost||!thehost.activereceive)
+            setactivereceive(index,getupdatedata()->allhosts+index,startthreads);
+        }
+    else  {
+        if(thehost.activereceive) {
+            if(thehost.activereceive==active_receivenr)
+                --active_receivenr;
+              thehost.activereceive=0;
+            }
         }
-    else
-        thehost.activereceive=0;
         
     shouldaskfordata=getshouldaskfordata();
-#ifdef WEAROS_MESSAGES
-extern    void clearnetworkcache();
+    #ifdef WEAROS_MESSAGES
+    extern    void clearnetworkcache();
     clearnetworkcache();
     #endif
     LOGGER("changehost=%d\n",ret);
     return ret;
     }
+
+
+
+
+
+
 bool isreceiving() const {
     const auto nr=getupdatedata()->hostnr;
     const passhost_t *host=getupdatedata()->allhosts;
@@ -996,7 +1028,6 @@ bool isreceiving() const {
             }
         }
     return receives;
-//    return getupdatedata()->hostnr!= getupdatedata()->sendnr;
     }
 
 void deactivateHost(int index,bool deactive) {
@@ -1007,44 +1038,47 @@ void deactivateHost(int index,bool deactive) {
 
     if(host.deactivated==deactive)
         return;
+#ifdef JUGGLUCO_APP
     if(host.wearos)
         setBlueMessage(index,false);
+#endif
     host.deactivated=deactive;
+    Connect *con=connections[index];
     if(deactive) {
+        if(con) {
+                con->finish=true;
+                }
         if(host.activereceive) {
-            LOGGER("stop active receive     shutdown(%d)\n",hostsocks[index]);
-            extern std::vector<condvar_t*> active_receive;
-            active_receive[host.activereceive-1]->wakebackup(Backup::wakeend);
-            ::shutdown(hostsocks[index],SHUT_RDWR);
-            }
-        else {
-            if(host.receivefrom) {
-            int sock=hostsocks[index];
-            if(sock>=0) {
-                LOGGER("%d: close(%d)\n",index,sock);
-                    ::shutdown(sock,SHUT_RDWR);
-//             if(get_owner_tag(sock)==mytag()) 
-                     ::sockclose(sock);
-                    hostsocks[index]=-1;
-                    }
+            active_receive[host.activereceive-1]->wakebackup(wakeend);
+            if(con) {
+                LOGGER("stop active receive     shutdown(%d)\n",con->getReceiverIdent());
+                con->shutdownReceiver();
                 }
             }
         int sin=host.index;
         if(sin>=0) {
-               const int sock= getupdatedata()->tosend[sin].getsock();
-               LOGGER(" shutdown %d\n",sock);
                if(con_vars[sin])
-                con_vars[sin]->wakebackuponly(Backup::wakestop|Backup::wakeend);
-            ::shutdown(sock,SHUT_RDWR);
+                    con_vars[sin]->wakebackuponly(wakestop|wakeend);
+               if(con) {
+                   LOGGER(" shutdown %d\n",con->getSenderIdent());
+                   con->shutdownSender();
+                   }
             }
-
+        if(con)
+            con->endConnection();
         }
     else {
+        if(con) {
+                con->finish=false;
+                }
+      setConnectTime(index,0);
         if(host.index>=0)
             startthread(index,host.index);
         if(host.activereceive) {
             setactivereceive(index,&host);
-            }
+               }
+       if(host.ICE)
+            startReceiverThread(index);
         }
     }
 void notupdatedsettings() {
@@ -1063,22 +1097,18 @@ int getsendhostnr() const {
 //int histnr=0;
 //SensorGlucoseData *backuphist=nullptr;
 //static constexpr const uintptr_t wakereopen=1;
-static constexpr const uintptr_t wakestream=1;
-static constexpr const uintptr_t wakescan=2;
-static constexpr const uintptr_t wakenums=4;
-static constexpr const uintptr_t wakeall=8;
-static constexpr const uintptr_t wakestop=16;
-static constexpr const uintptr_t wakeother=32;
-static constexpr const uintptr_t wakeend=64;
-static constexpr const uintptr_t wakesend=128;
-static constexpr const uintptr_t wakereconnect=256;
-static constexpr const uintptr_t wakestreamsend=512;
 void closeallsocks() {
     LOGAR("closeallsocks");
     for(int i=0;i<getupdatedata()->hostnr;i++) {
-         LOGGER("hostsock %d shutdown(%d)\n",i,hostsocks[i]);
-        ::shutdown(hostsocks[i],SHUT_RDWR);
+        if(auto *con=connections[i]) {
+             LOGGER("closeall receiver %d shutdown(%d)\n",i,con->getReceiverIdent());
+            con->closeReceiverConnection();
+            LOGGER("closeall sender %d shutdown(%d)\n",i,con->getSenderIdent());
+            con->closeSenderConnection();
+            }
+//        ::shutdown(hostsocks[i],SHUT_RDWR);
         }
+        /*
     for(int i=0;i<getupdatedata()->sendnr;i++) {
         if(sendsocks[i]>=0) {
             LOGGER("send %d shutdown(%d)\n",i,sendsocks[i]);
@@ -1086,15 +1116,28 @@ void closeallsocks() {
             }
         else
            LOGGER("sendsock[%d]==-1\n",i);
-        }
+        } */
     }
+void endAllConnections() {
+    LOGAR("endAllConnections");
+    for(int i=0;i<getupdatedata()->hostnr;i++) {
+        if(auto *con=connections[i]) {
+            con->endConnection();
+            }
+        }
+     }
 void closesocksone(int allindex,passhost_t *host) {
     LOGGER("closesocksone %d\n",allindex);
+    if(Connect *connect=connections[allindex]) {
+        connect->shutdownReceiver();
+        connect->shutdownSender();
+        }
+    /*
     int sock=hostsocks[allindex];
     if(sock>=0) {
         hostsocks[allindex]=-1;
         ::shutdown(sock,SHUT_RDWR);
-        }
+        } 
      int ind=host->index;
      if(ind>=0) {
         int ssock=sendsocks[ind];
@@ -1106,33 +1149,55 @@ void closesocksone(int allindex,passhost_t *host) {
         else
            LOGGER("sendsock[%d]==-1\n",ind);
         }
+        */
     }
 bool sendwakesender(int h) {
+    LOGGER("sendwakesender(%d)\n",h);
     updateone &shost=getupdatedata()->tosend[h];
     shost.close();
     shost.open();
-    return sendbackup(shost.getcrypt(),shost.getsock());
+    if(auto *con=shost.getConnect()) {
+        if(con->finish) {
+            LOGGER("sendwakesenser: %d finish\n",h);
+            return false;
+            }
+        if(con->allindex!=shost.allindex) {
+                LOGGER("sendwakesender %d allindex different\n",h);
+                return false;
+                }
+        return con->sendbackup(shost.getcrypt());
+        }
+    LOGAR("sendwakesender shost.getConnect()==null");
+     return false;
     }
 
 bool sendwakestreamsender(int h) {
+    LOGGER("sendwakestreamsender(%d)\n",h);
     updateone &shost=getupdatedata()->tosend[h];
     shost.close();
     shost.open();
-    extern    bool sendwakeupstream(crypt_t *pass,const int sock);
-    return sendwakeupstream(shost.getcrypt(),shost.getsock());
+    if(auto *con=shost.getConnect()) {
+        if(con->finish) {
+            LOGGER("sendwakestreamsender: %d finish\n",h);
+            return false;
+            }
+        if(con->allindex!=shost.allindex) {
+                LOGGER("sendwakestreamsender %d allindex different\n",h);
+                return false;
+                }
+        return con->sendwakeupstream(shost.getcrypt());
+        }
+    LOGAR("sendwakestreamsender shost.getConnect()==null");
+     return false;
     }
-    /*
-extern int updateproc(condvar_t *varsptr,uintptr_t cond,updateone &shost,int  (updateone::*proc)( )) ;
- extern void    endbackupthread(int h) ;
-extern void        doupdates(const uintptr_t current,const int h) ;
-extern void lockwait(uintptr_t &current,int h) ;
-extern void        notpassive(uintptr_t current,int sendindex);
-extern bool doend(int sendindex); */
+
+
 void backupthread(int allindex,int sendindex) {
 #ifndef NOLOG
     auto &host=getupdatedata()->tosend[sendindex];
+   auto *con=connections[allindex];
+    LOGGER("%d backupthread, wearos=%d con_vars=%p sock=%i %p\n", allindex,getupdatedata()->allhosts[allindex].wearos, con_vars[sendindex],con?con->getSenderIdent():-1,host.getcrypt());
 #endif
-    LOGGER("%d backupthread, wearos=%d con_vars=%p sock=%i %p\n", allindex,getupdatedata()->allhosts[allindex].wearos,con_vars[sendindex],host.getsock(),host.getcrypt());
 //    const int sendindex=getupdatedata()->allhosts[allindex].index;
     const bool passive=getupdatedata()->allhosts[allindex].sendpassive;
 
@@ -1152,6 +1217,10 @@ void backupthread(int allindex,int sendindex) {
        while(true) {
           if(doend(sendindex)) 
             return;
+        if(!con_vars[sendindex]) { 
+                LOGGER("con_vars[%d]==null\n",sendindex);
+                return;
+                }
          if(!con_vars[sendindex]->dobackup) {
             status.locked=true;
             lockwait(current,sendindex);
@@ -1176,6 +1245,7 @@ void backupthread(int allindex,int sendindex) {
 
 void        notpassive(uintptr_t current,int sendindex) {
         if(current&wakereconnect)  {
+            LOGGER("notpassive(%ul,%d) wakereconnect\n",current,sendindex);
             getupdatedata()->tosend[sendindex].close();
             getupdatedata()->tosend[sendindex].open();
             } 
@@ -1188,11 +1258,12 @@ void        notpassive(uintptr_t current,int sendindex) {
     }
 bool doend(int sendindex) {
     const condvar_t* var=con_vars[sendindex];
-    LOGGER("doend: con_vars=%p\n",var);
       if(!var||(var->dobackup&wakeend))  {
+         LOGGER("doend: con_vars=%p end\n",var);
         endbackupthread(sendindex);
         return true;
         }
+    LOGGER("doend: con_vars=%p continue\n",var);
     return false;
     }
 
@@ -1216,12 +1287,15 @@ int updateproc(condvar_t *varsptr,uintptr_t cond,updateone &shost,int  (updateon
                 return 0;
             shost.open();
               if(varsptr->dobackup&wakestop)   {
+                LOGAR("updateproce wakestop");
                 shost.close();
                 return 0;
                 }
             res= (shost.*proc)();
-            if(!res)
+            if(!res) {
+                LOGAR("updateproce !res");
                 shost.close();
+                }
             }
         return res;
            }
@@ -1260,16 +1334,26 @@ void        doupdates(const uintptr_t current,const int h) {
             (didscans=        updateproc(con_vars[h],current&wakescan,getupdatedata()->tosend[h],&updateone::updatescansu))&&
             (didupdate=        updateproc(con_vars[h],current&wakeall,getupdatedata()->tosend[h],&updateone::update));
         const uint16_t command=((didnums&1)?wakenums:0)|(didstream&1?wakestream:0)|(didscans&1?wakescan:0)|(didupdate&1?wakeall:0);
-        if(command)
-            sendrender(getupdatedata()->tosend[h].getcrypt(),getupdatedata()->tosend[h].getsock(),command);
+        if(command) {
+            auto &host=getupdatedata()->tosend[h];
+            host.getConnect()->sendrender(host.getcrypt(),command);
+            }
         }
 void lockwait(uintptr_t &current,int h) {
     LOGGER("%d before lock\n",h)    ;
-        std::unique_lock<std::mutex> lck(con_vars[h]->backupmutex);
-        con_vars[h]->dobackup=con_vars[h]->dobackup&~current;
-        con_vars[h]->backupcond.wait(lck, [h] {return backup->con_vars[h]->dobackup; });   
-        current=con_vars[h]->dobackup;
-      LOGGER("%d after lock sock=%d\n",h,getupdatedata()->tosend[h].getsock())    ;
+    std::unique_lock<std::mutex> lck(con_vars[h]->backupmutex);
+    LOGGER("%d after lock\n",h)    ;
+    con_vars[h]->dobackup=con_vars[h]->dobackup&~current;
+    LOGGER("%d dobackup=%d\n",h,con_vars[h]->dobackup)    ;
+    con_vars[h]->backupcond.wait(lck, [h] {return backup->con_vars[h]->dobackup; });   
+    LOGGER("%d afterwait\n",h)    ;
+    current=con_vars[h]->dobackup;
+    #ifndef NOLOG
+    LOGGER("%d after current=\n",h)    ;
+    int allindex=getupdatedata()->tosend[h].allindex;
+    auto *con=connections[allindex];
+   LOGGER("%d after connections[%d]=%p eSenderIdent=%d\n",h,allindex,con,con?con->getSenderIdent():-1)    ;
+   #endif
     }
 
 //void streambackup(int index) { wakebackup(wakestream); }
@@ -1345,6 +1429,9 @@ static void startbackup(std::string_view globalbasedir) {
             auto &host=backup->getupdatedata()->allhosts[i];
             int index=host.index;
             LOGGER("index=%d\n",index);
+            if(host.ICE&&!host.deactivated) {
+                startReceiverThread(i);
+                }
             if(index>=maxsend) {
                 LOGGER("allhosts[i].index %d >= %d sendnr\n",index,maxsend);
                 host.index=-1;
@@ -1397,15 +1484,17 @@ inline std::span<passhost_t> getHosts() {
     }
 };
 
+inline std::span<passhost_t> getBackupHosts() {
+    return backup-> getHosts();
+    }
+inline int gethostindex(const passhost_t *host) {
+    return host-getBackupHosts().data();
+    }
+
 //extern void wakebackup(int kind=1);
 
 
 extern void startbackup(std::string_view globalbasedir) ;
 
 
-inline std::span<passhost_t> getBackupHosts() {
-    return backup-> getHosts();
-    }
-inline int gethostindex(passhost_t *host) {
-    return host-getBackupHosts().data();
-    }
+
