@@ -246,7 +246,7 @@ struct Tings {
         };
     int32_t shortnr;
     std::array<Shortcut,maxvarnr> shorts;
-    uint8_t  mealvar;
+    int8_t  mealvar;
     uint8_t  dexcomPredict;
     uint16_t nightsensor;
     float roundto;
